@@ -25,6 +25,7 @@
 
 #if PSYCH_SYSTEM == PSYCH_OSX
 double PsychCocoaGetBackingStoreScaleFactor(void* window);
+void PsychCocoaAssignCAMetalLayer(PsychWindowRecordType *windowRecord);
 #endif
 
 // Pointer to master onscreen window during setup phase of stereomode 10 (Dual-window stereo):
@@ -342,7 +343,7 @@ PsychError SCREENOpenWindow(void)
             if (stereomode == kPsychDualStreamStereo)
                 PsychErrorExitMsg(PsychError_user, "Use of VRR mode for fine-grained stimulus presentation timing is incompatible with dual-stream stereo presentation on special display devices. Choose either VRR or this stereo mode. Aborting.");
 
-            if (imagingmode & kPsychNeedFinalizedFBOSinks)
+            if ((imagingmode & kPsychNeedFinalizedFBOSinks) && !((PSYCH_SYSTEM == PSYCH_LINUX) && (vrrMode != kPsychVRROwnScheduled) && (specialflags & kPsychExternalDisplayMethod)))
                 PsychErrorExitMsg(PsychError_user, "Use of VRR mode for fine-grained stimulus presentation timing is incompatible with use of finalized FBO sinks on special display devices. Choose either VRR or finalized FBO sinks. Aborting.");
         }
 
@@ -955,6 +956,13 @@ PsychError SCREENOpenWindow(void)
                 windowRecord->internalMouseMultFactor = isf;
                 windowRecord->externalMouseMultFactor = 1.0;
             }
+
+            // Graphics api interop setup under Cocoa, e.g., for Vulkan MoltenVK interop.
+            // This is the point where we transition from OpenGL rendering and display to
+            // display via the external graphics consumer, ie. switching to the CAMetalLayer.
+            // OpenGL rendering to our onscreen window and OpenGL bufferswap will no longer
+            // work from here on, only OpenGL rendering to the interop FBO's/textures:
+            PsychCocoaAssignCAMetalLayer(windowRecord);
         }
         else {
             // CGL:
