@@ -1,6 +1,6 @@
 function osxmakeit(mode)
 % This is the MacOS/X version of makeit: It is meant for building PTB for
-% 64-Bit Matlab on OSX 10.11 "El Crapitan" and later, using the 10.12 SDK.
+% 64-Bit Matlab on OSX 10.15, using the 10.15 SDK.
 
 if ~IsOSX(1) || IsOctave
     error('osxmakeit only works with a 64-Bit version of Matlab for OSX!');
@@ -15,7 +15,7 @@ if mode == -1
     % Yes: Call ourselves recursively on all plugins/modes to rebuild
     % everything:
     tic;
-    for mode = 0:14
+    for mode = 0:15
         osxmakeit(mode);
     end
     elapsedsecs = toc;
@@ -57,7 +57,7 @@ if mode==0
         -U CoreDisplay_Display_SetUserBrightness -U CoreDisplay_Display_GetUserBrightness ...
         -U DisplayServicesCanChangeBrightness -U DisplayServicesBrightnessChanged ...
         -U CoreDisplay_Display_SetAutoBrightnessIsEnabled ...
-        -framework IOKit -framework SystemConfiguration -framework Carbon -framework Cocoa -framework CoreText" ...
+        -framework IOKit -framework SystemConfiguration -framework Carbon -framework Cocoa -framework CoreText -framework QuartzCore" ...
         CFLAGS="\$CFLAGS -x objective-c" -I/Library/Frameworks/GStreamer.framework/Versions/Current/include/gstreamer-1.0 ...
         -I/Library/Frameworks/GStreamer.framework/Versions/Current/include/libxml2 -I/Library/Frameworks/GStreamer.framework/Versions/Current/include/glib-2.0 ...
         -I/Library/Frameworks/GStreamer.framework/Versions/Current/lib/gstreamer-1.0/include/ ...
@@ -193,4 +193,24 @@ if mode==14
     unix(['mv ../Projects/MacOSX/build/PsychOculusVRCore.' mexext ' ' PsychtoolboxRoot 'PsychBasic/']);
 end
 
+if mode==15
+    % Build PsychVulkanCore:
+    % Depends on a system level (/usr/local/[share/include/lib]/
+    % installation of the Vulkan SDK and MoltenVK for macOS from
+    % https://vulkan.lunarg.com for prebuilt SDK and Vulkan ICD,
+    % https://github.com/KhronosGroup/MoltenVK for source code.
+    %
+    % We link directly to MoltenVK, as we use some MVK functions which are
+    % not yet part of the Vulkan standard, and thereby not supported by the
+    % Vulkan loader and layers, as of Mid-May 2021.
+    %
+    % We link to OpenGL to implement OpenGL-Vulkan interop.
+    try
+        mex -outdir ../Projects/MacOSX/build/ -output PsychVulkanCore -largeArrayDims -DMEX_DOUBLE_HANDLE -DPTBMODULE_PsychVulkanCore LDFLAGS="\$LDFLAGS -framework CoreServices -framework CoreFoundation -framework CoreAudio -framework OpenGL" -ICommon/Base -IOSX/Base -ICommon/PsychVulkanCore -I/usr/local/include "Common/PsychVulkanCore/*.c" "OSX/Base/*.c" "Common/Base/*.c" -lvulkan -lMoltenVK
+    catch
+        disp(psychlasterror);
+    end
+    unix(['mv ../Projects/MacOSX/build/PsychVulkanCore.' mexext ' ' PsychtoolboxRoot 'PsychBasic/']);
+end
+
 return;
