@@ -53,7 +53,7 @@ if nargin > 0 && isscalar(cmd) && isnumeric(cmd)
         tWhen = varargin{3};
 
         if varargin{4} == 0
-            doTimestamp = 1;
+            doTimestamp = 2; % High precision system-provided scheduling/timestamping if possible.
         else
             doTimestamp = 0;
         end
@@ -144,6 +144,69 @@ if nargin > 0 && isscalar(cmd) && isnumeric(cmd)
 
         return;
     end
+
+    % Special present code for macOS, to work around Apple's broken Metal
+    % implementation and other macOS bugs:
+    if cmd == 2
+        % Execute flip operation via a Vulkan Present operation at the appropriately
+        % scheduled requested visual stimulus onset time:
+        win = varargin{1};
+        vwin = varargin{2};
+        tWhen = varargin{3};
+
+        if varargin{4} == 0
+            doTimestamp = 2; % High precision system-provided scheduling/timestamping if possible.
+        else
+            doTimestamp = 0;
+        end
+
+        % Present and maybe get an ok precision and not too unreliable timestamp from Vulkan driver:
+        predictedOnset = PsychVulkanCore('Present', vwin, tWhen, doTimestamp);
+
+        % Get a second opinion on onset time from Screen's VBLANK timestamping:
+        winfo = Screen('GetWindowInfo', win, 7);
+
+        % Likely valid vblTime returned from Screen()?
+        if (winfo.VBLEndline > 0) && (winfo.LastVBLTime > 0)
+            % Yes. Assign:
+            vblTime = winfo.LastVBLTime;
+        else
+            % No. Fallback to GetSecs as a noisy last resort:
+            vblTime = GetSecs;
+        end
+
+        % If predictedOnset is valid, use it. Otherwise fall back to vblTime:
+        if predictedOnset > 0
+            % Valid timestamp from Vulkan? Validate a bit and warn if not:
+            if (verbosity > 4) || ((verbosity > 1) && (abs(predictedOnset - vblTime) > 0.001))
+                fprintf('PsychVulkan-DEBUG: Delta between Vulkan and reference timestamps is %f usecs.\n', 1e6 * (predictedOnset - vblTime));
+            end
+
+            vblTime = predictedOnset;
+        elseif predictedOnset == 0
+            % Vulkan timestamping returned bogus timestamp zero, but no diagnosed failure.
+            % Use what we got from Screen() or GetSecs():
+            predictedOnset = vblTime;
+
+            if verbosity > 1
+                fprintf('PsychVulkan-DEBUG: Vulkan timestamping failed. Falling back to reference timestamp %f secs. Timing or visual stimulation might be broken.\n', vblTime);
+            end
+        else
+            % Error code timestamp -1 returned. We can't recover in a
+            % meaningful way from that, just pass it through...
+            vblTime = -1;
+
+            if verbosity > 1
+                fprintf('PsychVulkan-DEBUG: Vulkan timestamping failed completely. Returning invalid timestamp -1.\n');
+            end
+        end
+
+        % Inject vblTime and visual stimulus onset time into Screen(), for usual handling
+        % and reporting back to usercode via Screen('Flip'), also current beamposition:
+        Screen('Hookfunction', win, 'SetOneshotFlipResults', '', vblTime, predictedOnset, [], winfo.Beamposition);
+
+        return;
+    end % Of macOS special code.
 end % Of fast-path dispatch.
 
 % Slow path dispatch:
@@ -209,7 +272,8 @@ if strcmpi(cmd, 'Supported')
     % Init supported flag via one-time probe:
     if isempty(supported)
         try
-            if exist('PsychVulkanCore', 'file') && PsychVulkanCore('GetCount') > 0
+            if exist('PsychVulkanCore', 'file') && (PsychVulkanCore('GetCount') > 0) && ...
+               (~IsOSX || IsMinimumOSXVersion(10,15,4)) % macOS 10.15.4 is the bare minimum needed.
                 supported = 1;
             else
                 supported = 0;
@@ -235,7 +299,7 @@ if strcmpi(cmd, 'OpenWindowSetup')
     outputName = varargin{1};
     screenId = varargin{2};
     winRect = varargin{3};
-    ovrfbOverrideRect = varargin{4}; %#ok<NASGU>
+    ovrfbOverrideRect = varargin{4};
     ovrSpecialFlags = varargin{5};
     if isempty(ovrSpecialFlags)
         ovrSpecialFlags = 0;
@@ -379,8 +443,10 @@ if strcmpi(cmd, 'OpenWindowSetup')
         Screen('Preference', 'ScreenToHead', screenId, outputMappings{screenId + 1}(1, outputIndex + 1), outputMappings{screenId + 1}(2, outputIndex + 1), 0);
     end
 
-    % These always have to match:
-    ovrfbOverrideRect = winRect;
+    if ~IsOSX
+        % These always have to match:
+        ovrfbOverrideRect = winRect;
+    end
 
     % Set ovrSpecialFlags override settings to mark the onscreen window as not
     % important for visual stimulation, because the actual window / OpenGL windowing
@@ -459,6 +525,11 @@ if strcmpi(cmd, 'PerformPostWindowOpenSetup')
     winfo = Screen('GetWindowInfo', win);
     screenId = Screen('WindowScreenNumber', win);
     refreshHz = Screen('Framerate', screenId);
+    if refreshHz == 0
+        % macOS reports refresh rate of 0 on Mac builtin panels - the idiocy...
+        refreshHz = 60;
+    end
+
     devs = PsychVulkanCore('GetDevices');
 
     % Restore rank 0 output setting in Screen:
@@ -483,20 +554,48 @@ if strcmpi(cmd, 'PerformPostWindowOpenSetup')
     end
 
     if isempty(strfind(glGetString(GL.EXTENSIONS), 'GL_EXT_memory_object')) %#ok<STREMP>
-        flags = mor(flags, 1);
-        noInterop = 1;
         % If no specific Vulkan gpu was requested, select the first non-AMD/NVidia
-        % gpuIndex (as we know AMD/NVidia can't be it - they fully support interop).
-        % Default to gpuIndex 1 in case this filtering fails to find an elegible gpu:
+        % gpuIndex on Linux or Windows: AMD and NVidia OpenGL fully support
+        % OpenGL interop, so if we end here then the render-gpu can not be an
+        % AMD or NVidia, ergo the display gpu should not be one.
+        %
+        % Also ignore MoltenVK (DriverId == 14) on macOS, because Apple's
+        % OpenGL does not support this extension at all, so we can't use
+        % this as selection criterion. Instead we just choose the first
+        % enumerated gpu on macOS: On a single-gpu machine that is the
+        % obviously correct choice. On a dual-gpu MacBookPro, the 1st gpu
+        % seems to be the discrete high-performance gpu, which matches
+        % Screen()'s choice of gpu as OpenGL renderer, so again gpuIndex 1
+        % would give us a match.
+        %
+        % Default also to gpuIndex 1 in case this filtering fails to find an eligible gpu:
         if isempty(gpuIndex) || gpuIndex == 0
             gpuIndex = 1;
             for i=1:length(devs)
-                if ~ismember(devs(i).DriverId, [1, 2, 3, 4])
+                if ~ismember(devs(i).DriverId, [1, 2, 3, 4, 14])
                     gpuIndex = devs(i).DeviceIndex;
                 end
             end
         end
-        fprintf('PsychVulkan-INFO: OpenGL implementation does not support OpenGL-Vulkan interop! Enabling basic diagnostic mode on gpu %i.\n', gpuIndex);
+
+        if IsOSX
+            noInterop = 0;
+
+            % Disable Direct-To-Display mode, it is buggy as hell, at least
+            % on macOS 10.15.7 with AMD Radeon Pro 560. Not that it works
+            % much better with this hack, it is only a bit better. At the
+            % same time, this hack supposedly adds one frame of extra
+            % latency, but our measurements show that even without it,
+            % there is one frame of extra latency, contrary to what the
+            % docs wrt. Direct-to-Display mode say. Broken stuff all
+            % around on the iToys operating system:
+            flags = mor(flags, 2);
+        else
+            flags = mor(flags, 1);
+            noInterop = 1;
+
+            fprintf('PsychVulkan-INFO: OpenGL implementation does not support OpenGL-Vulkan interop! Enabling basic diagnostic mode on gpu %i.\n', gpuIndex);
+        end
     else
         noInterop = 0;
     end
@@ -585,8 +684,15 @@ if strcmpi(cmd, 'PerformPostWindowOpenSetup')
             % TODO XXX: Should we calculate refreshHz per output or from FlipInterval instead?
         end
     else
-        % On Windows, outputHandle is meaningless atm.:
-        outputHandle = uint64(0);
+        if IsOSX
+            % On macOS we need the CAMetalLayer backing the onscreen window in
+            % kPsychExternalDisplayMethod mode. It is stored in SysWindowInteropHandle:
+            outputHandle = uint64(winfo.SysWindowInteropHandle);
+        else
+            % On Windows, outputHandle is meaningless atm.:
+            outputHandle = uint64(winfo.SysWindowHandle);
+        end
+
         if isFullscreen
             % Mark output 0 (the only possible output for a screenId on
             % non-Linux/X11) of screenId as used:
@@ -663,7 +769,7 @@ if strcmpi(cmd, 'PerformPostWindowOpenSetup')
         % No interop, or semaphores unsupported?
         if noInterop || isempty(strfind(glGetString(GL.EXTENSIONS), 'GL_EXT_semaphore')) %#ok<STREMP>
             if ~noInterop
-                fprintf('PsychVulkan-INFO: OpenGL implementation does not support OpenGL-Vulkan interop semaphores! Enabling operation without semaphores on gpu %i.\n', gpuIndex);
+                fprintf('PsychVulkan-INFO: OpenGL implementation does not support OpenGL-Vulkan interop semaphores. Enabling operation without semaphores on gpu %i.\n', gpuIndex);
             else
                 fprintf('PsychVulkan-INFO: Interop disabled! Enabling operation without semaphores on gpu %i.\n', gpuIndex);
             end
@@ -739,7 +845,12 @@ if strcmpi(cmd, 'PerformPostWindowOpenSetup')
 
     % Set it up:
     if ~noInterop
-        Screen('Hookfunction', win, 'ImportDisplayBufferInteropMemory', [], 0, interopObjectHandle, allocationSize, internalFormat, tilingMode, memoryOffset, width, height, renderCompleteSemaphore);
+        if IsOSX
+            oldtex = Screen('Hookfunction', win, 'SetDisplayBufferTextures', [], double(interopObjectHandle), [], GL.TEXTURE_RECTANGLE, internalFormat, 0, width, height);
+            glDeleteTextures(1, oldtex); % Get rid of now unused old texture.
+        else
+            Screen('Hookfunction', win, 'ImportDisplayBufferInteropMemory', [], 0, interopObjectHandle, allocationSize, internalFormat, tilingMode, memoryOffset, width, height, renderCompleteSemaphore);
+        end
     end
 
     vulkan{win}.valid = 1;
@@ -783,12 +894,28 @@ if strcmpi(cmd, 'PerformPostWindowOpenSetup')
     Screen('Hookfunction', win, 'SetOneshotFlipFlags', '', kPsychDontAutoResetOneshotFlags + kPsychSkipWaitForFlipOnce + kPsychSkipSwapForFlipOnce + kPsychSkipTimestampingForFlipOnce);
 
     if ~noglfinish
-        % Old method as fallback: Use glFinish to sync Vulkan with OpenGL:
-        Screen('Hookfunction', win, 'AppendMFunction', 'LeftFinalizerBlitChain', 'Vulkan Mono commit operation', 'moglcore(''glFinish'');');
+        % Old method as fallback: Use glFinish to sync Vulkan with OpenGL, except on macOS where glFlush seems enough:
+        if IsOSX
+            Screen('Hookfunction', win, 'AppendMFunction', 'LeftFinalizerBlitChain', 'Vulkan Mono commit operation', 'moglcore(''glFlush'');');
+        else
+            Screen('Hookfunction', win, 'AppendMFunction', 'LeftFinalizerBlitChain', 'Vulkan Mono commit operation', 'moglcore(''glFinish'');');
+        end
         Screen('Hookfunction', win, 'Enable', 'LeftFinalizerBlitChain');
     end
 
-    cmdString = sprintf('PsychVulkan(0, %i, %i, IMAGINGPIPE_FLIPTWHEN, IMAGINGPIPE_FLIPVBLSYNCLEVEL);', win, vwin);
+    if IsOSX
+        % Special present path for Apples broken macOS:
+        if ~vulkan{win}.SupportsTiming
+            % We are screwed on macOS:
+            sca;
+            error('PsychVulkan-ERROR: macOS Vulkan does not provide builtin timing support. Game over!');
+        end
+
+        cmdString = sprintf('PsychVulkan(2, %i, %i, IMAGINGPIPE_FLIPTWHEN, IMAGINGPIPE_FLIPVBLSYNCLEVEL);', win, vwin);
+    else
+        % Well working operating systems:
+        cmdString = sprintf('PsychVulkan(0, %i, %i, IMAGINGPIPE_FLIPTWHEN, IMAGINGPIPE_FLIPVBLSYNCLEVEL);', win, vwin);
+    end
     Screen('Hookfunction', win, 'AppendMFunction', 'PreSwapbuffersOperations', 'Vulkan Present operation', cmdString);
     Screen('Hookfunction', win, 'Enable', 'PreSwapbuffersOperations');
 
