@@ -78,7 +78,7 @@ if IsOctave
             end
             
             if IsLinux
-                fprintf('The following directory should be the *first one* on your Octave path:\n %s \n\n', [PsychtoolboxRoot 'PsychBasic/Octave3LinuxFiles' oext]);
+                %fprintf('The following directory should be the *first one* on your Octave path:\n %s \n\n', [PsychtoolboxRoot 'PsychBasic/Octave3LinuxFiles' oext]);
             end
             if IsOSX
                 fprintf('The following directory should be the *first one* on your Octave path:\n %s \n\n', [PsychtoolboxRoot 'PsychBasic/Octave6OSXFiles' oext]);
@@ -90,7 +90,7 @@ if IsOctave
             % Correct file with correct extension, still load failure:
             % Check for supported Octave version:
             fprintf('Your version of Octave (%s) might be incompatible with Psychtoolbox: We support Octave 4.0.0 or later\n', version);
-            fprintf('on Linux, Octave 6.1 on Windows and macOS.\n');
+            fprintf('on Linux, Octave 6.1 and 6.2 on Windows and macOS.\n');
 
             fprintf('Another reason could be some missing 3rd party dynamic link shared libraries on your system.\n');
             fprintf('Another reason could be some binary incompatibility. You would need to recompile Psychtoolbox from source!\n\n');
