@@ -5,10 +5,16 @@ function FlipTimingWithRTBoxPhotoDiodeTest(configFile, targetFolder, usevulkan,
 % timestamping precision and robustness under varying loads, conditions and
 % modes of operation. This requires one of the supported external
 % measurement devices to provide the "ground truth" for true stimulus onset
-% times. Currently supported: UCST RTBox with photo-diode, VPixx Inc. DataPixx/
-% ViewPixx/ProPixx, UBW32/Bitwhacker + UCST VideoSwitcher. The script can also
-% be used without external equipment to just test stimulus onset accuracy with
-% only indirect test of timestamping.
+% times. Currently supported:
+% - UCST RTBox with its own photo-diode (p)
+% - VPixx Inc. DataPixx/ViewPixx/ProPixx (d)
+% - UBW32/Bitwhacker + UCST VideoSwitcher (b)
+% - UCST Videoswitcher + UCST RtBox or CRS Bits# emulated RtBox (v)
+% - Other TTL trigger timestamp emitting devices + UCST RtBox or CRS Bits#,
+%   e.g., the CRS-ColorCal2 used as a photo-diode. (p)
+%
+% The script can also be used without external equipment (n) to just test
+% stimulus onset accuracy with only indirect test of timestamping.
 %
 % This documentation is incomplete for now, good luck!
 %
@@ -23,7 +29,8 @@ function FlipTimingWithRTBoxPhotoDiodeTest(configFile, targetFolder, usevulkan,
 % the users home directory is tried as target.
 %
 % 'usevulkan' If 1, try to use a Vulkan display backend instead of the
-% OpenGL display backend. See 'help PsychVulkan'.
+% OpenGL display backend. See 'help PsychVulkan' for supported hardware +
+% operating system combinations and required setup.
 %
 % 'bpc' Request a specific output framebuffer color precision. Currently
 % supported are 8 for standard 8 bpc RGBA8 framebuffer, 10 bpc for RGB10A2,
@@ -243,6 +250,10 @@ if IsLinux && (conf.VBLTimestampingMode == 1)
     conf.VBLTimestampingMode = 4;
 end
 
+if IsOSX && (conf.VBLTimestampingMode == 1)
+    conf.VBLTimestampingMode = 0;
+end
+
 %conf.VBLTimestampingMode = 3;
 %res.VBLTimestampingMode = conf.VBLTimestampingMode;
 % VBLTimestampingMode: configVBLTimestampingMode
@@ -335,6 +346,18 @@ try
     % Number of trials to perform:
     n = size(conf.waitFramesSched, 2);
 
+    if IsOSX && usevulkan
+        % Work around macOS + Vulkan/Metal scheduling flaws. 5 ifi's is the
+        % lowest we can hope to achieve with macOS 10.15.7 + MoltenVK 1.1.1
+        % + Metal as of March 2021, so play it safe with 6 as a minimum:
+        minallowed = 6;
+        minwaitFrames = min(conf.waitFramesSched);
+        if minwaitFrames < minallowed
+            minwaitFrames = minallowed - minwaitFrames;
+            conf.waitFramesSched = conf.waitFramesSched + minwaitFrames;
+        end
+    end
+
     % Init data-collection arrays for collection of n samples:
     res.rawFlipTime     = zeros(1, n);
     res.finishFlipTime  = zeros(1, n);
@@ -381,7 +404,7 @@ try
     end
 
     % Hide the cursor!
-    HideCursor;
+    HideCursor(w);
 
     % Perform some initial Flip to get us in sync with retrace:
     % tvbl is the timestamp (system time in seconds) when the retrace
@@ -390,6 +413,13 @@ try
     Screen('FillRect', w, 0);
     tvbl=Screen('Flip', w);
 
+    if tvbl <= 0 && IsOSX && usevulkan
+        % Work around macOS Vulkan/Metal bugs: Sporadic invalid timestamps
+        % need to be replaced with something reasonable to avoid flip
+        % failure:
+        tvbl = GetSecs;
+    end
+
     WaitSecs(1);
 
     if useRTbox ~= -1000
@@ -409,8 +439,8 @@ try
             % Clear receive buffers to start clean:
             PsychRTBox('Clear', rtbox);
 
-            if ~IsWin
-                % Hack: Enable async background reads to speedup box operations:
+            if ~IsWin && ~(IsOSX && usevulkan)
+                % Hack: Enable async background reads to speedup box operations on high quality systems:
                 IOPort ('ConfigureSerialport', res.boxinfo.handle, 'BlockingBackgroundRead=1 StartBackgroundRead=1');
             end
         else
@@ -526,6 +556,14 @@ try
         end
 
         res.vblFlipTime(i) = tvbl;
+
+        if tvbl <= 0 && IsOSX && usevulkan
+            % Work around macOS Vulkan/Metal bugs: Sporadic invalid timestamps
+            % need to be replaced with something reasonable to avoid flip
+            % failure:
+            tvbl = GetSecs;
+        end
+
         winfo = Screen('GetWindowInfo', w);
         res.rawFlipTime(i) = winfo.RawSwapTimeOfFlip;
         % This is flips internal timestamp of when the Swapbuffers call is
@@ -562,6 +600,20 @@ try
                     res.failFlag(i) = 0;
                     res.measuredTime(i) = mytstamp;
                 end
+
+                if IsOSX && usevulkan
+                    % Current macOS 10.15.7 Metal will not give us low
+                    % enough flip latency to flip back to black within one
+                    % video refresh cycle due to system compositor design
+                    % flaws. The "double-exposure" to two frames with
+                    % trigger lines with the Videoswitcher would screw up
+                    % our data, so make sure to clear the wrong value out.
+                    % This will ofc. add even more latency - no winning
+                    % here. But at least the measurements we can get won't
+                    % be wrong:
+                    PsychRTBox('Clear', rtbox);
+                end
+
                 PsychRTBox('EngageLightTrigger', rtbox);
                 PsychRTBox('EngagePulseTrigger', rtbox);
             else
@@ -644,9 +696,6 @@ try
     % Shutdown realtime scheduling:
     finalprio = Priority(0) %#ok<NASGU,NOPRT>
 
-    % Show the cursor:
-    ShowCursor;
-
     % Need to perform remapping on Datapixx before Screen('CloseAll'):
     if useRTbox == -1
         % Datapixx: Remap timestamps to GetSecs time:
@@ -659,7 +708,7 @@ try
     % Close display: If we skipped/missed any presentation deadline during
     % Flip, Psychtoolbox will automatically display some warning message on the Matlab
     % console:
-    Screen('CloseAll');
+    sca;
     WaitSecs(1);
 
     if useRTbox ~= -1000
@@ -667,7 +716,7 @@ try
             PsychRTBox('Stop', rtbox);
             PsychRTBox('Clear', rtbox);
 
-            if ~IsWin
+            if ~IsWin && ~(IsOSX && usevulkan)
                 % Hack: Disable async background reads:
                 fprintf('Stopping background read op on box...\n');
                 IOPort ('ConfigureSerialport', res.boxinfo.handle, 'StopBackgroundRead');
@@ -695,6 +744,15 @@ try
         end
     end
 
+    % On macOS + Vulkan, deal with invalid timestamps returned. Replace
+    % with NaN's so they don't disturb the analysis and plots:
+    if IsOSX && usevulkan
+        validmacOS = res.vblFlipTime > 0;
+        res.vblFlipTime(~validmacOS) = nan;
+        res.onsetFlipTime(~validmacOS) = nan;
+        fprintf('Warning: Replaced %i invalid Flip timestamps on macOS Vulkan / Metal with NaN.\n', n - sum(validmacOS));
+    end
+
     % Store results to filesystem before we start shutdown and plotting:
     save(res.outFilename, 'res', '-V6');
     res.outFilename = [];
@@ -749,7 +807,7 @@ try
         figure;
     end
 
-    valids = (res.failFlag == 0);
+    valids = (res.failFlag == 0) & (res.onsetFlipTime > 0);
     fprintf('Measured samples %i [realvalid %i, corrupted %i] vs. Flip samples %i.\n', length(res.measuredTime), length(find(res.failFlag == 0)), length(find(res.failFlag == 1)), i);
     res.measuredTime = res.measuredTime(valids);
     res.onsetFlipTime = res.onsetFlipTime(valids);
@@ -804,14 +862,14 @@ catch
     % This "catch" section executes in case of an error in the "try" section
     % above. Importantly, it closes the onscreen window if its open and
     % shuts down realtime-scheduling of Matlab:
-    ShowCursor;
-    Screen('CloseAll');
-
     if ~isempty(res.outFilename)
         % Store results to filesystem before we start shutdown and plotting:
         save(res.outFilename, 'res', '-V6');
     end
 
+    % Close window, restore lut's, show cursor, etc.:
+    sca;
+
     % Disable realtime-priority in case of errors.
     Priority(0);
 
